/* fsl.cpp
 * written by brent ledvina
 * 12.8.2000
 */

#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>  
#include <sys/ioctl.h>
#include <string.h>
#include <ctype.h>
#include "otherstructs.h"
#include "fsl.h"
#include "ephem.h"

#define DEBUG 0
#define wr 0


FSL::FSL()
{
  FSLfile = -1;
  FSLfifo = -1;
  outfile = -1;

  fsllogfile.Active = FALSE;
  fsllogfile.RecordsLogged = 0;
}

FSL::FSL(int fifo)
{
  FSLfile = -1;
  FSLfifo = fifo;

  fsllogfile.Active = FALSE;
  fsllogfile.RecordsLogged = 0;
}

FSL::~FSL()
{

}

int FSL::setFifo(int fifo)
{
  FSLfifo = fifo;

  return 0;
}


int FSL::GenerateFilename(Date *d, char *filename, int i)
{
  sprintf(filename,"%02d%02d%02d%c%1d.fsl",d->year,d->month,d->day,fsllogfile.obsinfo->SiteID,i);
  return 0;
}


int FSL::SetupHeader(ObserverInfo *obsinfo)
{
  this->obsinfo = *obsinfo;
  
  this->fsllogfile.obsinfo = &this->obsinfo;
  
  return 0;
}



int FSL::OpenFSLFile(char *filename)
{
  
  if((FSLfile = open(filename,O_RDONLY)) < 0)
  {
    //fprintf(stderr,"fsl: Error opening file %s\n",filename);
    return -1;
  }
  return 0;
}


int FSL::OpenFile(Date *d)
{
  char filename[13];  
  int i = 0;
  GenerateFilename(d,filename,i);
  
  // check if the file is there
  while((FSLfile = open(filename,O_WRONLY|O_EXCL|O_CREAT,S_IRUSR|S_IROTH)) < 0)
  {
    if(errno == EEXIST)
      GenerateFilename(d,filename,++i);
    else
      fprintf(stderr,"Error opening FSL file\n");
   
  }
  
  strcpy(fsllogfile.Filename,filename);
    
  return FSLfile;
}

int FSL::CloseFile()
{
  if(FSLfile > 0)
    close(FSLfile);

  return 0;
}

int FSL::StopLogging()
{
  return 0;
}

int FSL::ReSync()
{
  unsigned char byte;
  int n;

  // standard entry has the synchronization bits in the msb of
  // the timeTIC. This routine searches for the bit pattern byte
  // by byte.
  
  while((n = read(FSLfile,&byte,1)) > 0){
    if((byte & 0x7F) == (SYNC_MSB>>24)) {
      fprintf(stdout,"re-synchronized!\n");
      lseek(FSLfile,-1,SEEK_CUR);
      return 1;
    }
  }
  return 0;
}

int FSL::ReadFSLData(int *SVs, unsigned int *time, unsigned int *svmarkers, fslsatstruc3 *fslstuff, fslsatstruc3 *noise, int *noisepresent, int flag)
{
  char buffer[100];
  int n,nflag;
  int i;
  fslsatstruc3 fsldata[MAXCHANNELS];
  fslsatstruc3 noiseentry;
    
  unsigned long svcurrent;     /* Bitmap of satellites used in solution. */
  unsigned long temp;
  unsigned long timetic;     
  unsigned char chksum;

  int sat, SVcount = 0;  
  int SV[MAXSATELLITES];

  //if(show)
  //  fprintf(stderr,"reading fsl data from file\n");
  if(flag == 0)
  {  
    if((n = read(FSLfile,&timetic,sizeof(unsigned int))) < 0)
    { 
      fprintf(stderr,"\nError reading time\n");
      return n;
    }		
    // Check for synchronization byte. 
    if ((timetic & 0x7F000000UL) != SYNC_MSB) {
      fprintf(stderr,"Lost Sync\n");
      //*status = SYNC_LOST;
      return SYNC_LOST;
    }
    if((n = read(FSLfile,&svcurrent,sizeof(unsigned int))) < 0) {
      fprintf(stderr,"\nError reading svcurrent\n");
      return n;
    } 
    // read in the chksum
    if((n = read(FSLfile,&chksum,sizeof(unsigned char))) < 0) {
      fprintf(stderr,"\nError reading checksum\n");
      return n;
    } 
    // need to verify the chksum too
     if (!CRCcheck(timetic,svcurrent,chksum)) {
       //*status = CRC_INVALID;
      fprintf(stderr,"CRC Error\n");
     return CRC_INVALID;
   }  

    // extract time from timetic 
    *time = timetic;
    *svmarkers = svcurrent;
  }
  
 // extract noise channel flag and TIC number. */
  nflag = ((*time & 0x80000000UL) == 0x80000000UL);
  //printf("nflag = %d\n",nflag);
  
  // satellite markers
  temp = *svmarkers;
  
  //if(wr)
  //{
  //  sprintf(buffer,"0 %10lu %10lu 0 0 0 0 0\r\n",timeTIC & SYNC_MASK,svcurrent);
  //  write(outfile,buffer,35);
  //}
    
  for(sat= 0; sat <MAXSATELLITES; sat++)
  {
    if((temp&0x1) == 1)
    {
      SV[SVcount] = sat+1;
      SVcount++;
    }
    temp >>= 1;
  }
 
  // read the noise entry if present???
  if(nflag) {
    n = read(FSLfile, &noiseentry,sizeof(fslsatstruc3));
    *noise = noiseentry;
    *noisepresent = TRUE;
  }
  else
    *noisepresent = FALSE;

  if(SVcount > 0)
  {
    n = read(FSLfile, &fsldata[0],sizeof(fslsatstruc3)*SVcount);
    if(n == 0)  //finished reading data
      return 0;

    //if(wr)
    //{
    //  for(i=0;i<SVcount; i++)
    //  {
    //    sprintf(buffer,"%2u %2d %18.6f %8d %8d %8d %8d %8d\r\n",SV[i],fsldata[i].epoch,fsldata[i].int_carrier_phase,fsldata[i].WBPk[0],fsldata[i].WBPk[1],fsldata[i].WBPk[2],fsldata[i].WBPk[3],fsldata[i].WBPk[4]);
	
    //	write(outfile,buffer,71);
    //  }
    //}
    //if(show)
    //  fprintf(stderr,"timetic=%u svcurrent=%u, chksum=%uc\n",timeTIC,svcurrent,chksum);
    //for(i=0;i<SVcount; i++)
    //{
      //if(show)
      //{
      // 	fprintf(stderr,"e=%d %18.6f %d %d %d %d %d\n",fsldata[i].epoch,fsldata[i].int_carrier_phase,fsldata[i].WBPk[0],fsldata[i].WBPk[1], fsldata[i].WBPk[2],fsldata[i].WBPk[3],fsldata[i].WBPk[4]);
      //}
    //}
    //if(show)
    //{
    //  fprintf(stderr,"read from %d channels\n",SVcount);
    //  fprintf(stderr,"errno=%d\n",errno);
    //}
  }

  *time = *time&SYNC_MASK;
  for(i=0; i<SVcount; i++)
  {
    fslstuff[i] = fsldata[i];		
    SVs[i] = SV[i];
  }
  return SVcount;  
}



/* ReadFSLData
 *
 * current version
 *
 */
int FSL::ReadFSLData(int *SVs, unsigned int *time, unsigned int *svmarkers0, unsigned int *svmarkers1, fslsatstruc4 *fslstuff, fslsatstruc4 *noise, int *noisepresent, int flag)
{
  char buffer[100];
  int n,nflag;
  int i;
  fslsatstruc4 fsldata[MAXCHANNELS];
  fslsatstruc4 noiseentry;
    
  unsigned long svcurrent0;     /* Bitmap of satellites used in solution. */
  unsigned long svcurrent1;
  unsigned long temp0;
  unsigned long temp1;
  unsigned long timetic;     
  unsigned char chksum;

  int sat, SVcount = 0;  
  int SV[MAXSATELLITES+2];

#if DEBUG
  fprintf(stderr,"flag=%d\n",flag);
#endif

  //fprintf(stderr,"flag=%d\n",flag);
  //fprintf(stderr,"time=%08x\n",time);

  if(flag == 0) {  
    if((n = read(FSLfile,&timetic,sizeof(unsigned int))) < 0)
    { 
      fprintf(stderr,"\nError reading time\n");
      return n;
    }		
    //fprintf(stderr,"timetic=%08x\n",timetic);

    // Check for synchronization byte. 
    if ((timetic & 0x7F000000UL) != SYNC_MSB) {
      fprintf(stderr,"Lost Sync\n");
      //*status = SYNC_LOST;
      return SYNC_LOST;
    }
    if((n = read(FSLfile,&svcurrent0,sizeof(unsigned int))) < 0) {
      fprintf(stderr,"\nError reading svcurrent0\n");
      return n;
    } 
    if((n = read(FSLfile,&svcurrent1,sizeof(unsigned int))) < 0) {
      fprintf(stderr,"\nError reading svcurrent1\n");
      return n;
    }
    // read in the chksum
    if((n = read(FSLfile,&chksum,sizeof(unsigned char))) < 0) {
      fprintf(stderr,"\nError reading checksum\n");
      return n;
    } 
    //fprintf(stderr,"checking CRC...");
    // need to verify the chksum too
     if (!CRCcheck(timetic,svcurrent0,svcurrent1,chksum)) {
       //*status = CRC_INVALID;
      fprintf(stderr,"CRC Error\n");
     return CRC_INVALID;
     }  
     //fprintf(stderr,"done\n");

    // extract time from timetic 
    *time = timetic;
    //fprintf(stderr,"2: *time=%08x\n",*time); 
    *svmarkers0 = svcurrent0;
    *svmarkers1 = svcurrent1;
  }
  
 // extract noise channel flag and TIC number. */
  nflag = ((*time & 0x80000000UL) == 0x80000000UL);
  //printf("nflag = %d\n",nflag);
  
  // satellite markers
  temp0 = *svmarkers0;
  temp1 = *svmarkers1;

  for(sat= 0; sat <MAXSATELLITES; sat++) {
    if((temp0&0x1) == 1) {
      SV[SVcount] = sat+1;
      SVcount++;
    }
    temp0 >>= 1;
  }
  for(sat= 0; sat <MAXSATELLITES; sat++) {
    if((temp1&0x1) == 1) {
      SV[SVcount] = sat+33;
      SVcount++;
    }
    temp1 >>= 1;
  }

  // check for errors
  if(SVcount>MAXCHANNELS) {
    fprintf(stderr,"SVCount=%d\n",SVcount);
    return SYNC_LOST;
  }
  
  // read the noise entry if present???
  if(nflag) {
    n = read(FSLfile, &noiseentry,sizeof(fslsatstruc4));
    *noise = noiseentry;
    *noisepresent = TRUE;
  }
  else
    *noisepresent = FALSE;

  if(SVcount > 0) {
    n = read(FSLfile, &fsldata[0],sizeof(fslsatstruc4)*SVcount);
    if(n == 0)  //finished reading data
      return 0;
  }

  //fprintf(stderr,"time=%0X\n",time);
  //fprintf(stderr,"*time=%0X\n",*time);

  *time = *time&SYNC_MASK;
  for(i=0; i<SVcount; i++) {
    fslstuff[i] = fsldata[i];		
    SVs[i] = SV[i];
  }
  return SVcount;  
}



int FSL::ReadFSLData(int *SVs, unsigned int *time, unsigned int *svmarkers, fslsatstruc0 *fslstuff, fslsatstruc0 *noise,int *noisepresent, int flag)
{
  int n,nflag;
  int i;
  fslsatstruc0 fsldata[MAXCHANNELS];
  fslsatstruc0 noiseentry;   
  unsigned int svcurrent;     /* Bitmap of satellites used in solution. */
  unsigned int temp;
  unsigned int timetic;     
  unsigned char chksum;

  int sat, SVcount = 0;  
  int SV[MAXSATELLITES];

  //if(show)
  //  fprintf(stderr,"reading fsl data from file\n");
  if(flag == 0)
  {  
    if((n = read(FSLfile,&timetic,sizeof(unsigned int))) < 0)
    { 
      fprintf(stderr,"\nError reading time\n");
      return n;
    }		
    // Check for synchronization byte. 
    if ((timetic & 0x7F000000UL) != SYNC_MSB) {
      fprintf(stderr,"Lost Sync\n");
      //*status = SYNC_LOST;
      return SYNC_LOST;
    }
    if((n = read(FSLfile,&svcurrent,sizeof(unsigned int))) < 0)
    {
      fprintf(stderr,"\nError reading svcurrent\n");
      return n;
    } 
    // read in the chksum
    if((n = read(FSLfile,&chksum,sizeof(unsigned char))) < 0)
    {
      fprintf(stderr,"\nError reading checksum\n");
      return n;
    } 
    // need to verify the chksum too
     if (!CRCcheck(timetic,svcurrent,chksum)) {
       //*status = CRC_INVALID;
      fprintf(stderr,"CRC Error\n");
     return CRC_INVALID;
   }  
    

   
      
    *time = timetic;
    *svmarkers = svcurrent;
  }
    
  // extract noise channel flag and TIC number. */
  nflag = ((*time & 0x80000000UL) == 0x80000000UL);

  // satellite markers
  temp = *svmarkers;
  

  //if(wr)
  //{
  //  sprintf(buffer,"0 %10lu %10lu 0 0 0 0 0\r\n",timeTIC & SYNC_MASK,svcurrent);
  //  write(outfile,buffer,35);
  //}
    
  for(sat= 0; sat <MAXSATELLITES; sat++)
  {
    if((temp&0x1) == 1)
      SV[SVcount++] = sat+1;
    temp = temp >> 1;
  }

  //read the noise entry if present???
  if(nflag) {
    n = read(FSLfile, &noiseentry,sizeof(fslsatstruc0));
    *noise = noiseentry;
    *noisepresent = TRUE;
  }
  else
    *noisepresent = FALSE;

  if(SVcount > 0)
  {
    n = read(FSLfile, &fsldata[0],sizeof(fslsatstruc0)*SVcount);
    if(n == 0)  //finished reading data
      return 0;

    //if(wr)
    //{
    //for(i=0;i<SVcount; i++)
    //{
    //  sprintf(buffer,"%2u %2d %18.6f %8d %8d %8d %8d %8d\r\n",SV[i],fsldata[i].epoch,fsldata[i].int_carrier_phase,fsldata[i].WBPk[0],fsldata[i].WBPk[1],fsldata[i].WBPk[2],fsldata[i].WBPk[3],fsldata[i].WBPk[4]);
	
    //write(outfile,buffer,71);
    //}
    //}
    //if(show)
    //  fprintf(stderr,"timetic=%u svcurrent=%X=%u,chksum=%u \n",timeTIC,svcurrent,svcurrent,chksum);
    //for(i=0;i<SVcount; i++)
    //{
      //if(show)
      //{
      //	fprintf(stderr,"e=%d %18.6f %d %d %d %d %d\n",fsldata[i].epoch,fsldata[i].int_carrier_phase,fsldata[i].WBPk[0],fsldata[i].WBPk[1], fsldata[i].WBPk[2],fsldata[i].WBPk[3],fsldata[i].WBPk[4]);
      //}
      //}
    //if(show)
    //{
    //  fprintf(stderr,"read from %d channels\n",SVcount);
    //  fprintf(stderr,"errno=%d\n",errno);
    //}
  }

  *time = *time&SYNC_MASK;

  for(i=0; i<SVcount; i++)
  {
    fslstuff[i] = fsldata[i];		
    SVs[i] = SV[i];
  }

  return SVcount;  
}

int FSL::CheckAvailability(int prn, int version, unsigned int *time,  unsigned int *svmarkers)
{
  int i,sat,n;
  unsigned int timetic;
  unsigned int svcurrent,prnmarker=0,temp;
  unsigned char chksum;
  int svcount = 0,jump,nflag=0;

  if((n = read(FSLfile,&timetic,sizeof(unsigned int))) < 0) { 
    fprintf(stderr,"\nError reading time\n");
    return n;
  }

#if DEBUG
  fprintf(stderr,"timetic=0x%X masked=0x%X\n",timetic,(timetic & 0x7F000000UL));
#endif

  // Check for synchronization byte. 
  if ((timetic & 0x7F000000UL) != SYNC_MSB) {
    fprintf(stderr,"Lost Sync\n");
    //*status = SYNC_LOST;
      return -1;
  }
  
  if((n = read(FSLfile,&svcurrent,sizeof(unsigned int))) < 0) {
    fprintf(stderr,"\nError reading svcurrent\n");
    return n;
  } 

  if((n = read(FSLfile,&chksum,sizeof(unsigned char))) < 0) {
    fprintf(stderr,"\nError reading checksum\n");
    return n;
  } 
  
  if(n == 0)  //all done
    return -1;

  //you will want to return the data
  *time = timetic;
  *svmarkers = svcurrent;

  // extract noise channel flag and TIC number. */
  nflag = ((timetic & 0x80000000UL) == 0x80000000UL);

  // generate prn marker
  prnmarker = 1 << (prn-1);
  temp = svcurrent;  

  if ((svcurrent&prnmarker)==0) 
  {
    for(sat= 0; sat <MAXSATELLITES; sat++) {
      if((temp&0x1) == 1)
        svcount++;
      temp = temp >> 1;
    }
    //version 0 
    if(version == FSL_VERSION0) {
      jump = sizeof(fslsatstruc0)*svcount;  
      if(nflag)
        jump+=sizeof(fslsatstruc0);
    }
    else if(version == FSL_VERSION3)
    { 
      jump = sizeof(fslsatstruc3)*svcount;  
      if(nflag)
        jump+=sizeof(fslsatstruc3);
    }
    //fprintf(stderr,"n=%d, jump = %d\n",n,jump);
    lseek(FSLfile,jump,SEEK_CUR);
    return FALSE;
  }
  else 
  {
    //lseek(FSLfile,-9,SEEK_CUR);
    //fprintf(stderr,"nojump\n");
    return TRUE;
  }

}


int FSL::CheckAvailability(int prn, int version, unsigned int *time, unsigned int *svmarkers0, unsigned int *svmarkers1)
{
  int i,sat,n;
  unsigned int timetic;
  unsigned int svcurrent0,svcurrent1,prnmarker=0,temp0,temp1;
  unsigned char chksum;
  int svcount = 0,jump,nflag=0;

  if((n = read(FSLfile,&timetic,sizeof(unsigned int))) < 0) { 
    fprintf(stderr,"\nError reading time\n");
    return n;
  }

#if DEBUG
  fprintf(stderr,"timetic=0x%X masked=0x%X\n",timetic,(timetic & 0x7F000000UL));
#endif

  // Check for synchronization byte. 
  if ((timetic & 0x7F000000UL) != SYNC_MSB) {
    fprintf(stderr,"Lost Sync\n");
    //*status = SYNC_LOST;
      return -1;
  }
  
  if((n = read(FSLfile,&svcurrent0,sizeof(unsigned int))) < 0) {
    fprintf(stderr,"\nError reading svcurrent0\n");
    return n;
  } 
  if((n = read(FSLfile,&svcurrent1,sizeof(unsigned int))) < 0) {
    fprintf(stderr,"\nError reading svcurrent1\n");
    return n;
  } 

  if((n = read(FSLfile,&chksum,sizeof(unsigned char))) < 0) {
    fprintf(stderr,"\nError reading checksum\n");
    return n;
  } 
  
  if(n == 0)  //all done
    return -1;

  //you will want to return the data
  *time = timetic;
  *svmarkers0 = svcurrent0;
  *svmarkers1 = svcurrent1;

  // extract noise channel flag and TIC number. */
  nflag = ((timetic & 0x80000000UL) == 0x80000000UL);

  // generate prn marker
  if(prn<=32)
    prnmarker = 1 << (prn-1);
  else
    prnmarker = 1 << (prn-33);
  temp0 = svcurrent0;  
  temp1 = svcurrent1;  

#if DEBUG
  fprintf(stderr,"svcurrent0=%X svcurrent1=%X prnmarker=%X\n",svcurrent0,svcurrent1,prnmarker);
#endif

  if((svcurrent0&prnmarker)==0 && (svcurrent1&prnmarker)==0) 
  {
    for(sat= 0; sat <MAXSATELLITES; sat++) {
      if((temp0&0x1) == 1)
        svcount++;
      temp0>>= 1;
      if((temp1&0x1) == 1)
        svcount++;
      temp1>>=1;
    }
      
    jump = sizeof(fslsatstruc4)*svcount;  
    if(nflag)
      jump+=sizeof(fslsatstruc4);
    
    //fprintf(stderr,"n=%d, jump = %d\n",n,jump);
    lseek(FSLfile,jump,SEEK_CUR);
    return FALSE;
  } else {
    //lseek(FSLfile,-9,SEEK_CUR);
    //fprintf(stderr,"nojump\n");
    return TRUE;
  }

}


/* Log
 *
 *
 */
int FSL::Log()
{
  
  int n,nflag;
  int i;

  fslsatstruc4 fsldata[MAXCHANNELS];
  
  unsigned long svcurrent0;     /* Bitmap of satellites used in solution. */
  unsigned long svcurrent1;     /* Bitmap of satellites used in solution. */
  unsigned long temp0,temp1;
  unsigned long timeTIC;     
  int sat, SVcount = 0;  
  unsigned char chksum = 0;

#if DEBUG
    fprintf(stderr,"reading fsl data\n");
#endif    

  n = read(FSLfifo,&timeTIC,sizeof(unsigned long));
  n = read(FSLfifo,&svcurrent0,sizeof(unsigned long));
  n = read(FSLfifo,&svcurrent1,sizeof(unsigned long));

  //is there a noise structure present?
  nflag = ((timeTIC & 0x80000000UL) == 0x80000000UL);

  // add byte sync
  timeTIC = SYNC_MSB | timeTIC;

  // generate a checksum
  CRCgen(timeTIC, svcurrent0, svcurrent1, &chksum);

  if(n > 0) {
    temp0 = svcurrent0;
    temp1 = svcurrent1;

    write(FSLfile,&timeTIC,sizeof(unsigned long));
    write(FSLfile,&svcurrent0,sizeof(unsigned long));
    write(FSLfile,&svcurrent1,sizeof(unsigned long));
    write(FSLfile,&chksum,sizeof(unsigned char));
    
    for(sat=0;sat<MAXSATELLITES;sat++) {
      if((temp0&0x1) == 1)
	SVcount++;
      temp0>>=1;

      if((temp1&0x1) == 1)
	SVcount++;
      temp1>>=1;
    }

     if(nflag) {
      //read the noise channel data
      n=read(FSLfifo,&fsldata[0],sizeof(fslsatstruc4));
      if(n<=0) {
	fprintf(stderr,"Error reading from fifo\n");
        return 0;
      }
      //generate the checksum
      Checksum(&fsldata[0]);
      // write out the data
      write(FSLfile,&fsldata[0],sizeof(fslsatstruc4));
    } 
     
    if(SVcount > 0) {
      n=read(FSLfifo,&fsldata[0],sizeof(fslsatstruc4)*SVcount);
      if(n<=0) {
	fprintf(stderr,"Error reading from fifo\n");
        return 0;
      }
#if DEBUG
	fprintf(stderr,"timetic=%lu svcurrent0=%lu svcurrent1=%lu \n",timeTIC & SYNC_MASK,svcurrent0,svcurrent1);
#endif      
      for(i=0; i<SVcount;i++) {
 	// generate a checksum and add to the structure's flag
	Checksum(&fsldata[i]);
	write(FSLfile,&fsldata[i],sizeof(fslsatstruc4));

#if DEBUG
	  fprintf(stderr,"f=%X e=%d %d %d %d %d %d\n",fsldata[i].flags,fsldata[i].epoch,fsldata[i].WBPk[0],fsldata[i].WBPk[1], fsldata[i].WBPk[2],fsldata[i].WBPk[3],fsldata[i].WBPk[4]);
#endif
      }
#if DEBUG
        fprintf(stderr,"read from %d channels\n",SVcount);
        fprintf(stderr,"errno=%d\n",errno);
#endif 
    }
  }
  fsllogfile.RecordsLogged++;
  
  return SVcount;  
}


int FSL::ReadHeader(fslhdrstruc4 *header)
{
  int n;

  if((n = read(FSLfile,header,sizeof(fslhdrstruc4))) < 0) {
    fprintf(stderr,"Error reading header\n");
    return -1;
  }
  return 0;
  
}


int FSL::ReadHeader(fslhdrstruc3 *header)
{
  int n;

  if((n = read(FSLfile,header,sizeof(fslhdrstruc3))) < 0)
  {
    fprintf(stderr,"Error reading header\n");
    return -1;
  }
  return 0;
  
}

int FSL::ReadHeader(fslhdrstruc0 *header)
{
  int n;
  short i;
  char b[80];
  double d;
  float f;
  //if((n = read(FSLfile,header,sizeof(fslhdrstruc0))) < 0)
  //{
  //  fprintf(stderr,"Error reading header\n");
  //  return n;
  //}
  //lseek(FSLfile,0,SEEK_SET);

  //read each element by hand
  read(FSLfile,&i,sizeof(short int));  header->FSLversion = i;
  //printf("\ni=%d\n",i);
  read(FSLfile,b,21); strcpy(header->pgm,b);
  //printf("b=%s\n",b);
  read(FSLfile,b,21);strcpy(header->run_by,b);
  //printf("b=%s\n",b); 
  read(FSLfile,b,21);strcpy(header->date,b);
  //printf("b=%s\n",b); 
  read(FSLfile,b,61);strcpy(header->comment,b);
  //printf("b=%s\n",b); 
  read(FSLfile,b,61);strcpy(header->marker_name,b);
  //printf("b=%s\n",b);
  read(FSLfile,b,21);strcpy(header->observer,b);
  //printf("b=%s\n",b);
  read(FSLfile,b,21);strcpy(header->agency,b);
  //printf("b=%s\n",b);
  read(FSLfile,b,21);strcpy(header->rec_no,b);
  //printf("b=%s\n",b); 
  read(FSLfile,b,21);strcpy(header->rec_type,b);
  //printf("b=%s\n",b); 
  read(FSLfile,b,21);strcpy(header->rec_vers,b);
  //printf("b=%s\n",b); 
  read(FSLfile,b,21);strcpy(header->ant_no,b);
  //printf("b=%s\n",b);
  read(FSLfile,b,21);strcpy(header->ant_type,b);
  //printf("b=%s\n",b);
  read(FSLfile,&d,sizeof(double)); header->x = d;
  //printf("d=%f\n",d); header->x = d;
  read(FSLfile,&d,sizeof(double)); header->y = d;
  //printf("d=%f\n",d);
  read(FSLfile,&d,sizeof(double)); header->z = d;
  //printf("d=%f\n",d);
  read(FSLfile,&f,sizeof(float));header->delta_h = f;
  //printf("f=%f\n",f);
  read(FSLfile,&f,sizeof(float));header->delta_e = f;
  //printf("f=%f\n",f);
  read(FSLfile,&f,sizeof(float));header->delta_n = f;
  //printf("f=%f\n",f);
  read(FSLfile,&i,sizeof(short int));header->yy = i;
  //printf("i=%d\n",i);
  read(FSLfile,&i,sizeof(short int));header->mm = i;
  //printf("i=%d\n",i);
  read(FSLfile,&i,sizeof(short int));header->dd = i;
  //printf("i=%d\n",i);
  read(FSLfile,&i,sizeof(short int));header->h = i;
  //printf("i=%d\n",i);
  read(FSLfile,&i,sizeof(short int));header->m = i;
  //printf("i=%d\n",i);
  read(FSLfile,&d,sizeof(double));header->s = d;
  //printf("d=%f\n",d);


  //printf("sizeof(fslhdr0)=%d, n=%d\n",sizeof(fslhdrstruc0),n);
  //printf("SEEKING back 4 bytes\n");
  //lseek(FSLfile,-4,SEEK_CUR);
  return 0;
  
}


void FSL::printHeader(fslhdrstruc0 &hdr)
{
  fprintf(stderr,"ver=%d,pgm=%s,run_by=%s,date=%s\n",hdr.FSLversion,hdr.pgm,hdr.run_by,hdr.date);
  fprintf(stderr,"comment=%s,marker=%s,observer=%s,agency=%s\n",hdr.comment,hdr.marker_name,hdr.observer,hdr.agency);
  fprintf(stderr,"rec_no=%s,rec_type=%s,rec_vers=%s,ant_no=%s\n",hdr.rec_no,hdr.rec_type,hdr.rec_vers,hdr.ant_no);
  fprintf(stderr,"ant_type=%s,x=%f,y=%f,z=%f\n",hdr.ant_type,hdr.x,hdr.y,hdr.z);
  fprintf(stderr,"dh=%f,de=%f,dn=%f\n",hdr.delta_h,hdr.delta_e,hdr.delta_n);
  fprintf(stderr,"y=%d,m=%d,d=%d,h=%d,m=%d,s=%f\n",hdr.yy,hdr.mm,hdr.dd,hdr.h,hdr.m,hdr.s);

 return;
}

void FSL::printHeader(fslhdrstruc3 &hdr)
{
  fprintf(stderr,"ver=%d,pgm=%s,run_by=%s,date=%s\n",hdr.FSLversion,hdr.pgm,hdr.run_by,hdr.date);
  fprintf(stderr,"comment=%s,marker=%s,observer=%s,agency=%s\n",hdr.comment,hdr.marker_name,hdr.observer,hdr.agency);
  fprintf(stderr,"rec_no=%s,rec_type=%s,rec_vers=%s,ant_no=%s\n",hdr.rec_no,hdr.rec_type,hdr.rec_vers,hdr.ant_no);
  fprintf(stderr,"ant_type=%s,x=%f,y=%f,z=%f\n",hdr.ant_type,hdr.x,hdr.y,hdr.z);
  fprintf(stderr,"dh=%f,de=%f,dn=%f\n",hdr.delta_h,hdr.delta_e,hdr.delta_n);
  fprintf(stderr,"y=%d,m=%d,d=%d,h=%d,m=%d,s=%f\n",hdr.yy,hdr.mm,hdr.dd,hdr.h,hdr.m,hdr.s);

 return;
}

void FSL::printHeader(fslhdrstruc4 &hdr)
{
  fprintf(stderr,"version=%d\npgm=%s\nrun_by=%s\ndate=%s\n",hdr.FSLversion,hdr.pgm,hdr.run_by,hdr.date);
  fprintf(stderr,"comment=%s\nmarker=%s\nobserver=%s\nagency=%s\n",hdr.comment,hdr.marker_name,hdr.observer,hdr.agency);
  fprintf(stderr,"rec_no=%s\nrec_type=%s\nrec_vers=%s\nant_no=%s\n",hdr.rec_no,hdr.rec_type,hdr.rec_vers,hdr.ant_no);
  fprintf(stderr,"ant_type=%s\nx=%f y=%f z=%f\n",hdr.ant_type,hdr.x,hdr.y,hdr.z);
  
  fprintf(stderr,"dh=%f de=%f dn=%f\n",hdr.delta_h,hdr.delta_e,hdr.delta_n);
  fprintf(stderr,"year=%d\nmonth=%d\nday=%d\nhour=%d\nminute=%d\nsec=%f\n",hdr.yy,hdr.mm,hdr.dd,hdr.h,hdr.m,hdr.s);
  
  return;
}



// read the fsl version.
// the first 2 or 4 bytes contain the version
// dending on if it's version 0 or version 3, respectively
int FSL::ReadVersion(int &version)
{
  int n;
  int vs;

  if((n = read(FSLfile,&vs,sizeof(int))) < 0)
  {
    fprintf(stderr,"Error reading header\n");
    return -1;
  }
#if DEBUG
  fprintf(stderr,"vs = %X",vs);
#endif

  if(vs == FSL_VERSION4)
    version = FSL_VERSION4;
  else if(vs == FSL_VERSION3)
    version = FSL_VERSION3;
  else if((vs << 16) == FSL_VERSION0)
    version = FSL_VERSION0;
  else 
    version = -1;

  //reset file pointer to start of the file
  lseek(FSLfile, 0, SEEK_SET);

  return 0;
  
}


int FSL::WriteHeader(Date *d, Time *t,double *x, double *y, double *z)
{
  //char StationCom[] = "Phase Scintillation Study - Jan 2001";
  //char StationMrk[] = "Cachoiera Paulista, Brazil";
  
  fslhdrstruc4 header;
  double gsec, ss;

  static char datestr[21];

  //set the date and time
  fsllogfile.date = *d;
  fsllogfile.time = *t;

  // fix to make year 4 digits
  if(fsllogfile.date.year < 70)
    fsllogfile.date.year += 2000;	

  // build the header
  header.FSLversion = FSL_VERSION;

  sprintf(header.pgm,"CASCADE %s", CONFIG_CODE);
  header.pgm[20] = '\0';
  sprintf(header.run_by,fsllogfile.obsinfo->Observer);
  header.run_by[20] = '\0';
  sprintf(datestr,"%2d/%2d/%4d",fsllogfile.date.day,fsllogfile.date.month,fsllogfile.date.year);
  //for(i=3; i<6; i++) 
  //  datestr[i]=toupper(datestr[i]);
  strcpy(header.date, datestr);
  header.date[20] = '\0';
  strncpy(header.comment,fsllogfile.obsinfo->Comment,60);
  header.comment[60] = '\0';
  strncpy(header.marker_name,fsllogfile.obsinfo->StationMark,60);
  header.marker_name[60] = '\0';
  strncpy(header.observer,fsllogfile.obsinfo->Observer,20);
  header.observer[20] = '\0'; 
  strncpy(header.agency,fsllogfile.obsinfo->Agency,40);
  header.agency[20] = '\0';
  strncpy(header.rec_no,fsllogfile.obsinfo->ReceiverNumber,20);
  header.rec_no[20] = '\0';
  strncpy(header.rec_type,fsllogfile.obsinfo->ReceiverType,20);
  header.rec_type[20] = '\0';
  sprintf(header.rec_vers,"VER C%s",fsllogfile.obsinfo->ReceiverVersion);
  header.rec_vers[20] = '\0';
  strncpy(header.ant_no,fsllogfile.obsinfo->AntNumber,20);
  header.ant_no[20] = '\0';
  strncpy(header.ant_type,fsllogfile.obsinfo->AntType,20);
  header.ant_type[20] = '\0';

  header.x = *x;
  header.y = *y;
  header.z = *z;

  //antenna position
  header.delta_h = 0.0;
  header.delta_e = 0.0;
  header.delta_n = 0.0;
  
  header.yy = fsllogfile.date.year;
  header.mm =  fsllogfile.date.month;
  header.dd =  fsllogfile.date.day;
  header.h =  fsllogfile.time.hour;
  header.m =  fsllogfile.time.minute;
  header.s =  fsllogfile.time.second;
    
  //write the header
  write(FSLfile,&header,sizeof(header));
  
  return 0;
}




void FSL::CRCgen(unsigned int t, unsigned int s0, unsigned int s1, unsigned char *crc)
{
  static union
  {
    struct
    {    
      unsigned int sf0;
      unsigned int sf1;
      unsigned int tm;
    } hdr;
    unsigned char byte[8];
  }bytes;

  static const unsigned char CRCMASK = 0x1D;
  unsigned char check, data;
  int i, j;

  bytes.hdr.tm = t;
  bytes.hdr.sf0 = s0;
  bytes.hdr.sf1 = s1;

  check = bytes.byte[7];

  for(i=7;i>=0; i--)
  {
    if(i > 0)
      data = bytes.byte[i-1];
    else
      data =0x00;

    for(j=0; j< 8; j++) {
      if(check & 0x80)
	check = ((check << 1) | ((data & 0x80) == 0x80)) ^ CRCMASK;
      else
	check = ((check << 1) | ((data & 0x80) == 0x80));
      data <<= 1;
    }
  }
  *crc = check;
}


void FSL::CRCgen(unsigned int t, unsigned int s, unsigned char *crc)
{
  static union
  {
    struct
    {    
      unsigned int sf;
      unsigned int tm;
    } hdr;
    unsigned char byte[8];
  }bytes;

  static const unsigned char CRCMASK = 0x1D;
  unsigned char check, data;
  int i, j;

  bytes.hdr.tm = t;
  bytes.hdr.sf = s;

  check = bytes.byte[7];

  for(i=7;i>=0; i--)
  {
    if(i > 0)
      data = bytes.byte[i-1];
    else
      data =0x00;

    for(j=0; j< 8; j++)
    {
      if(check & 0x80)
	check = ((check << 1) | ((data & 0x80) == 0x80)) ^ CRCMASK;
      else
	check = ((check << 1) | ((data & 0x80) == 0x80));
      data <<= 1;
    }
  }
  *crc = check;
}


int FSL::CRCcheck(unsigned int t, unsigned int s0, unsigned int s1, unsigned char crc) {
   static union {
      struct {
         unsigned int sf0;
	 unsigned int sf1;
         unsigned int tm;
      } hdr;
      unsigned char byte[8];
   } bytes;

   static const unsigned char CRCMASK = 0x1D;
   unsigned char check, data;
   int i, j;

   bytes.hdr.tm = t;
   bytes.hdr.sf0 = s0;
   bytes.hdr.sf1 = s1;

   check = bytes.byte[7];

   for (i=7;i>=0;i--) {
      if (i>0)
         data = bytes.byte[i-1];
      else
         data = crc;

      for (j=0;j<8;j++) {
         if (check & 0x80)
            check = ((check << 1) | ((data & 0x80) == 0x80)) ^ CRCMASK;
         else
            check = (check << 1) | ((data & 0x80) == 0x80);
         data <<= 1;
      }
   }
   return (check == 0);
}


int FSL::CRCcheck(unsigned int t, unsigned int s, unsigned char crc) {
   static union {
      struct {
	unsigned int sf;
         unsigned int tm;
      } hdr;
      unsigned char byte[8];
   } bytes;

   static const unsigned char CRCMASK = 0x1D;
   unsigned char check, data;
   int i, j;

   bytes.hdr.tm = t;
   bytes.hdr.sf = s;

   check = bytes.byte[7];

   for (i=7;i>=0;i--) {
      if (i>0)
         data = bytes.byte[i-1];
      else
         data = crc;

      for (j=0;j<8;j++) {
         if (check & 0x80)
            check = ((check << 1) | ((data & 0x80) == 0x80)) ^ CRCMASK;
         else
            check = (check << 1) | ((data & 0x80) == 0x80);
         data <<= 1;
      }
   }
   return (check == 0);
}



void FSL::Checksum(fslsatstruc4 *rec)
{
  #define NUM_BYTES4 28 //FSL version 4 num. bytes per record ????
  
  static union {
    fslsatstruc4 sat;
    unsigned char byte[NUM_BYTES4];
  } data;
    
  unsigned char chk = 0;
  int i;

  data.sat = *rec;

  /* generate simple checksum (relies on PC byte order)*/
  for(i=0; i < (NUM_BYTES4-1);i++) {
    chk = (unsigned char)((chk + data.byte[i]) % 256);
  }
  rec->flags |= ((unsigned int) chk) << 8;
}



void FSL::Checksum(fslsatstruc3 *rec)
{
  #define NUM_BYTES3 22 //FSL version 3 num. bytes per record

  static union
  {
    fslsatstruc3 sat;
    unsigned char byte[NUM_BYTES3];
  } data;
    
  unsigned char chk = 0;
  int i;

  data.sat = *rec;

  /* generate simple checksum (relies on PC byte order)*/
  for(i=0; i < (NUM_BYTES3-1);i++)
  {
    chk = (unsigned char)((chk + data.byte[i]) % 256);
  }
  rec->flags |= ((unsigned int) chk) << 8;
}


void FSL::Checksum(fslsatstruc0 *rec)
{
  #define NUM_BYTES0 48  // FSL version 0 num bytes per record
  
  static union
  {
    fslsatstruc0 sat;
    unsigned char byte[NUM_BYTES0];
  } data;
    
  unsigned char chk = 0;
  int i;

  data.sat = *rec;

  /* generate simple checksum (relies on PC byte order)*/
  for(i=0; i < (NUM_BYTES0-1);i++)
  {
    chk = (unsigned char)((chk + data.byte[i]) % 256);
  }
  rec->flags |= ((unsigned int) chk) << 8;
}











