//fslstructs.h 

#ifndef __FSLSTRUCTS_H
#define __FSLSTRUCTS_H

#include "mytime.h"
#include "defines.h"

#define TRUE 1
#define FALSE 0
#define MAXSATELLITES 32
#define MAXCHANNELS 12
#define SYNC_MSB 0x57000000UL

#ifndef SYNC_MASK
#define SYNC_MASK 0xFFFFFFUL
#endif

#define FSL_VERSION 4
#define FSL_VERSION0 0            //FSL version number.
#define FSL_VERSION3 3            //FSL version number.
#define FSL_VERSION4 4            //FSL version number.
#define C_VERSION "1.10 (031016)"
#define CONFIG_CODE "1.10"

#define SCINT_WAIT 2
#define SCINT_STOP 3

typedef struct
{
  double int_carrier_phase;            /* Carrier beat phase (cycles). */
  unsigned short int WBPk[5];           /* Correlator based wide band power. */
  unsigned int gpstime;
  unsigned short int epoch;           /* Time of first C/No estimator sample. */
  unsigned short int flags;          /* Checksum and flags: e.g., lost lock. */
  
} fslsatstruc;

typedef struct
{
  double int_carrier_phase;            /* Carrier beat phase (cycles). */
  unsigned short int WBPk[5];           /* Correlator based wide band power. */
  unsigned int gpstime;
  unsigned short int epoch;           /* Time of first C/No estimator sample. */
  unsigned short int flags;          /* Checksum and flags: e.g., lost lock. */
  
} fslsatstruc4;


typedef struct
{
    //double pseudorange;                              /* Pseudorange (m). */
    //double range_rate;                        /* Pseudorange rate (m/s). */
    double int_carrier_phase;            /* Carrier beat phase (cycles). */
  //unsigned int NBPk[5];         /* Correlator based narrow band power. */
    unsigned short int WBPk[5];           /* Correlator based wide band power. */
    unsigned short int epoch;           /* Time of first C/No estimator sample. */
    unsigned short int flags;          /* Checksum and flags: e.g., lost lock. */
  
} fslsatstruc3;

typedef struct
{
    double pseudorange;                              /* Pseudorange (m). */
    double range_rate;                        /* Pseudorange rate (m/s). */
    double int_carrier_phase;            /* Carrier beat phase (cycles). */
    unsigned short NBPk[5];         /* Correlator based narrow band power. */
    unsigned short WBPk[5];           /* Correlator based wide band power. */
    unsigned short epoch;           /* Time of first C/No estimator sample. */
    unsigned short flags;          /* Checksum and flags: e.g., lost lock. */
  
} fslsatstruc0;

/* Set up special structures for data read from files. */
/* Header information for scintillation log file. */
/* Note: This header is the same header as in version 3.*/

typedef struct {
   int FSLversion;                               /* FSL version number. */
   char pgm[21];                             /* Program name for RINEX. */
   char run_by[21];                        /* Run by listing for RINEX. */
   char date[21];                                    /* Date for RINEX. */
   char comment[61];                              /* Comment for RINEX. */
   char marker_name[61];                      /* Marker Name for RINEX. */
   char observer[21];                    /* Observer listing for RINEX. */
   char agency[21];                        /* Agency listing for RINEX. */
   char rec_no[21];                       /* Receiver number for RINEX. */
   char rec_type[21];                       /* Receiver type for RINEX. */
   char rec_vers[21];                    /* Receiver version for RINEX. */
   char ant_no[21];                        /* Antenna number for RINEX. */
   char ant_type[21];                        /* Antenna type for RINEX. */
   double x;                                   /* X-position for RINEX. */
   double y;                                   /* Y-position for RINEX. */
   double z;                                   /* Z-position for RINEX. */
   float delta_h;                          /* Antenna height for RINEX. */
   float delta_e;                            /* Antenna east for RINEX. */
   float delta_n;                           /* Antenna north for RINEX. */
   int yy;                                  /* Starting year for RINEX. */
   int mm;                                 /* Starting month for RINEX. */
   int dd;                                   /* Starting day for RINEX. */
   int h;                                   /* Starting hour for RINEX. */
   int m;                                 /* Starting minute for RINEX. */
   double s;                              /* Starting second for RINEX. */
} fslhdrstruc4;


/* Set up special structures for data read from files. */
/* Header information for scintillation log file. */
typedef struct {
   int FSLversion;                               /* FSL version number. */
   char pgm[21];                             /* Program name for RINEX. */
   char run_by[21];                        /* Run by listing for RINEX. */
   char date[21];                                    /* Date for RINEX. */
   char comment[61];                              /* Comment for RINEX. */
   char marker_name[61];                      /* Marker Name for RINEX. */
   char observer[21];                    /* Observer listing for RINEX. */
   char agency[21];                        /* Agency listing for RINEX. */
   char rec_no[21];                       /* Receiver number for RINEX. */
   char rec_type[21];                       /* Receiver type for RINEX. */
   char rec_vers[21];                    /* Receiver version for RINEX. */
   char ant_no[21];                        /* Antenna number for RINEX. */
   char ant_type[21];                        /* Antenna type for RINEX. */
   double x;                                   /* X-position for RINEX. */
   double y;                                   /* Y-position for RINEX. */
   double z;                                   /* Z-position for RINEX. */
   float delta_h;                          /* Antenna height for RINEX. */
   float delta_e;                            /* Antenna east for RINEX. */
   float delta_n;                           /* Antenna north for RINEX. */
   int yy;                                  /* Starting year for RINEX. */
   int mm;                                 /* Starting month for RINEX. */
   int dd;                                   /* Starting day for RINEX. */
   int h;                                   /* Starting hour for RINEX. */
   int m;                                 /* Starting minute for RINEX. */
   double s;                              /* Starting second for RINEX. */
} fslhdrstruc3;

typedef struct {
   short int FSLversion;                         /* FSL version number. */
   char pgm[21];                             /* Program name for RINEX. */
   char run_by[21];                        /* Run by listing for RINEX. */
   char date[21];                                    /* Date for RINEX. */
   char comment[61];                              /* Comment for RINEX. */
   char marker_name[61];                      /* Marker Name for RINEX. */
   char observer[21];                    /* Observer listing for RINEX. */
   char agency[21];                        /* Agency listing for RINEX. */
   char rec_no[21];                       /* Receiver number for RINEX. */
   char rec_type[21];                       /* Receiver type for RINEX. */
   char rec_vers[21];                    /* Receiver version for RINEX. */
   char ant_no[21];                        /* Antenna number for RINEX. */
   char ant_type[21];                        /* Antenna type for RINEX. */
   double x;                                   /* X-position for RINEX. */
   double y;                                   /* Y-position for RINEX. */
   double z;                                   /* Z-position for RINEX. */
   float delta_h;                          /* Antenna height for RINEX. */
   float delta_e;                            /* Antenna east for RINEX. */
   float delta_n;                           /* Antenna north for RINEX. */
   short int yy;                                  /* Starting year for RINEX. */
   short int mm;                                 /* Starting month for RINEX. */
   short int dd;                                   /* Starting day for RINEX. */
   short int h;                                   /* Starting hour for RINEX. */
   short int m;                                 /* Starting minute for RINEX. */
   double s;                              /* Starting second for RINEX. */
} fslhdrstruc0;

#endif
