#define FSL_VERSION 2            //FSL version number.

/* Constants for FSL read routines. */
#define SYNC_MSB 0x57000000UL
#define SYNC_MASK 0x00FFFFFFUL
#define FLAG_MASK 0x00FF

#define NUM_BYTES 22                      /* no. bytes in fslsatstruc. */

/* Set up special structures for data read from files. */
/* Header information for scintillation log file. */
typedef struct {
	int FSLversion;                               /* FSL version number. */
	char pgm[21];                             /* Program name for RINEX. */
	char run_by[21];                        /* Run by listing for RINEX. */
	char date[21];                                    /* Date for RINEX. */
	char comment[61];                              /* Comment for RINEX. */
	char marker_name[61];                      /* Marker Name for RINEX. */
	char observer[21];                    /* Observer listing for RINEX. */
	char agency[21];                        /* Agency listing for RINEX. */
	char rec_no[21];                       /* Receiver number for RINEX. */
	char rec_type[21];                       /* Receiver type for RINEX. */
	char rec_vers[21];                    /* Receiver version for RINEX. */
	char ant_no[21];                        /* Antenna number for RINEX. */
	char ant_type[21];                        /* Antenna type for RINEX. */
	double x;                                   /* X-position for RINEX. */
	double y;                                   /* Y-position for RINEX. */
	double z;                                   /* Z-position for RINEX. */
	float delta_h;                          /* Antenna height for RINEX. */
	float delta_e;                            /* Antenna east for RINEX. */
	float delta_n;                           /* Antenna north for RINEX. */
	int yy;                                  /* Starting year for RINEX. */
	int mm;                                 /* Starting month for RINEX. */
	int dd;                                   /* Starting day for RINEX. */
	int h;                                   /* Starting hour for RINEX. */
	int m;                                 /* Starting minute for RINEX. */
	double s;                              /* Starting second for RINEX. */
} fslhdrstruc;

/* Satellite entry in scintillation log file. */
typedef struct
{	//Ver. FSL 2, reduced structure size
	double int_carrier_phase;              /* Carrier beat phase (cycles). */
	unsigned int WBPk[5];             /* Correlator based wide band power. */
	unsigned int epoch;             /* Time of first SNR estimator sample. */
	unsigned int flags;            /* Checksum and flags: e.g., lost lock. */
} fslsatstruc;

/* Function prototypes. */
int CRCcheck(unsigned long t, unsigned long s, unsigned char crc);
int ChecksumCheck(fslsatstruc *rec);

